from django.conf import settings
from django.shortcuts import render
from django.views.generic import ListView, View

from .mixins import FilterByQueryParamsMixin


class HandyHelperBaseListView(FilterByQueryParamsMixin, ListView):
    """ base view for CBV list pages

    class parameters:
         queryset - queryset to be rendered on the page
         title - title to use in template
         page_description - subtitle to use in template
         table - htm file rendering the queryset to be included in the generic_list template
         modals - htm file rendering additional modals to be included in the generic_list template

    example:
        class ListProjects(HandyHelperBaseListView):
            queryset = Project.objects.all()
            title = "Projects"
            page_description = "my cool projects"
            table = "table/table_projects.htm"
            modals = "project_modals.htm"
    """
    base_template = settings.BASE_TEMPLATE
    template = "handyhelpers/generic/generic_list.html"
    title = None
    table = None
    modals = None

    def get(self, request, *args, **kwargs):
        context = dict()
        template = self.template
        context['base_template'] = self.base_template
        context['queryset'] = self.filter_by_query_params()
        context['title'] = self.title
        context['sub_title'] = self.page_description
        context['table'] = self.table
        context['modals'] = self.modals
        return render(request, template, context=context)


class HandyHelperBaseCreateListView(FilterByQueryParamsMixin, ListView):
    """ base view for CBV list pages that include create form

    class parameters:
        queryset - queryset to be rendered on the page
        title - title to use in template
        page_description - subtitle to use in template
        table - htm file rendering the queryset to be included in the generic_list template
        modals - htm file rendering additional modals to be included in the generic_list template
        create_form_obj = form object
        create_form_url = url the form (action) should post to
        create_form_title = title to use on the create form (can be html)
        create_form_modal = name of modal for the create form
        create_form_link_title = text used for the link opening the create form

    example:
        class ListProjects(HandyHelperBaseCreateListView):
            queryset = Project.objects.all()
            title = "Projects"
            page_description = "my cool projects"
            table = "table/table_projects.htm"
            create_form_obj = ProjectForm
            create_form_url = '/hostmgr/create_project/'
            create_form_title = "<b>Add Project: </b><small> </small>"
            create_form_modal = "add_project"
            create_form_link_title = "add project"
    """
    base_template = settings.BASE_TEMPLATE
    template = "handyhelpers/generic/generic_list.html"
    title = None
    table = None
    modals = None
    create_form = dict()
    create_form_obj = None
    create_form_url = None
    create_form_title = None
    create_form_modal = None
    create_form_link_title = None

    def get(self, request, *args, **kwargs):
        context = dict()
        template = self.template
        context['base_template'] = self.base_template
        context['queryset'] = self.filter_by_query_params()
        context['title'] = self.title
        context['sub_title'] = self.page_description
        context['table'] = self.table
        context['modals'] = self.modals
        if self.create_form_obj:
            self.create_form['form'] = self.create_form_obj(request.user.username, request.POST or None)
            self.create_form['action'] = "Add"
            self.create_form['action_url'] = self.create_form_url
            self.create_form['title'] = self.create_form_title
            self.create_form['modal_name'] = self.create_form_modal
            self.create_form['link_title'] = self.create_form_link_title
            context['create_form'] = self.create_form
        return render(request, template, context=context)


class HandyHelperAboutView(View):
    """
    Generic view to render an 'about' page.

    class parameters:
        title    - page title to use ('About' if not provided)
        subtitle - subtitle to use (None if not provided)
        version  - version to display (None if not provided)
        details  - any specific details to display (None if not provided)
        source   - link to source code repository (None if not provided)
        contact  - any desired contact information (None if not provided)
        links    - list of dictionary containing links; ex. [{'some reference': 'www.somewebsite.com}, ...]
    """
    base_template = settings.BASE_TEMPLATE
    template = "handyhelpers/generic/generic_about.html"
    title = "About"
    subtitle = None
    version = None
    details = None
    source = None
    contact = None
    links = []

    def get(self, request, *args, **kwargs):
        context = dict()
        template = self.template
        context['base_template'] = self.base_template
        context['title'] = self.title
        context['subtitle'] = self.subtitle
        context['version'] = self.version
        context['details'] = self.details
        context['source'] = self.source
        context['contact'] = self.contact
        context['links'] = self.links
        return render(request, template, context=context)



"""
Description:
    Collection of handy generic views

"""

import csv
from django.http import HttpResponse
from django.shortcuts import render
from django.views.generic import (View)
from django.utils import timezone
import datetime
import logging


class SimpleView(View):
    """ super simple view for displaying a page with no queried data """
    template_name = None
    data = dict()

    def get(self, request):
        return render(request, self.template_name, self.data)


class CsvExportView(View):
    """ dump a queryset to a csv file """
    queryset = None
    filename = None

    def get(self, request):
        try:
            model = self.queryset.model
            if not self.filename:
                self.filename = "{}.csv".format(model._meta.model_name)
            response = HttpResponse(content_type='text/csv')
            cd = 'attachment; filename="{0}"'.format(self.filename)
            response['Content-Disposition'] = cd
            headers = [field.name for field in model._meta.fields]
            writer = csv.DictWriter(response, fieldnames=headers)
            writer.writeheader()
            for row in self.queryset:
                writer.writerow({column: str(getattr(row, column)) for column in headers})
            return response
        except AttributeError:
            return HttpResponse(content_type='text/csv')


class FilterByQueryParamsMixin:
    """ """
    request = None
    queryset = None
    page_description = None

    def filter_by_query_params(self):
            """
            Description:
                Filters a queryset by query_params in the URL

            Returns:
                filtered queryset
            """
            # pass a description to the view if included as a query parameter
            if not self.page_description and 'page_description' in self.request.GET.dict():
                self.page_description = self.request.GET.dict().get('page_description', None)

            # build a dictionary of valid model fields
            filter_dict = {}
            model = self.queryset.model
            for field, val in self.request.GET.dict().items():
                if field.split("__")[0] not in [i.name for i in model._meta.fields + model._meta.many_to_many]:
                    continue
                if val is not None:
                    if val == 'None':
                        val = None
                    filter_dict[field] = val
            return self.queryset.filter(**filter_dict)


class AnnualTrendView(View):
    """
    Description:
        Tallies the number of elements added/removed over the past year. Included are counts of elements added and
        removed in the past day, week, month and year.
        Also includes counts of elements added and removed by month for the past year.
    """
    title = "Annual Trend Report"
    sub_title = None
    template_name = None
    added_queryset = None
    removed_queryset = None
    added_query_field = None
    removed_query_field = None

    def get(self, request):
        context = dict()
        context['title'] = self.title
        context['sub_title'] = self.sub_title
        last_day = timezone.now() - datetime.timedelta(days=1)
        last_week = timezone.now() - datetime.timedelta(days=7)
        last_month = timezone.now() - datetime.timedelta(days=365.2425 / 12)
        last_year = timezone.now() - datetime.timedelta(days=365.2425)

        added_query_field = self.added_query_field + "__gte"
        removed_query_field = self.removed_query_field + "__gte"

        added_day = self.added_queryset.filter(**{added_query_field: last_day})
        removed_day = self.removed_queryset.filter(**{removed_query_field: last_day})
        added_week = self.added_queryset.filter(**{added_query_field: last_week})
        removed_week = self.removed_queryset.filter(**{removed_query_field: last_week})
        added_month = self.added_queryset.filter(**{added_query_field: last_month})
        removed_month = self.removed_queryset.filter(**{removed_query_field: last_month})
        added_year = self.added_queryset.filter(**{added_query_field: last_year})
        removed_year = self.removed_queryset.filter(**{removed_query_field: last_year})

        context['month_labels'] = []
        context['added_per_month'] = []
        context['removed_per_month'] = []
        for i in range(0, 13):
            ts = timezone.now() - datetime.timedelta(i * 365.2425 / 12)
            context['month_labels'].append(ts.strftime("%B"))
            try:
                context['added_per_month'].append(len(
                    [i for i in added_year if getattr(i, self.added_query_field).month == ts.month and
                     getattr(i, self.added_query_field).year == ts.year]))
            except Exception as err:
                logging.error(err)
                context['added_per_month'].append(0)
            try:
                context['removed_per_month'].append(len(
                    [i for i in removed_year if getattr(i, self.removed_query_field).month == ts.month and
                     getattr(i, self.removed_query_field).year == ts.year]))
            except Exception as err:
                logging.error(err)
                context['removed_per_month'].append(0)

        context['added_day'] = added_day.count()
        context['removed_day'] = removed_day.count()
        context['added_week'] = added_week.count()
        context['removed_week'] = removed_week.count()
        context['added_month'] = added_month.count()
        context['removed_month'] = removed_month.count()
        context['added_year'] = added_year.count()
        context['removed_year'] = removed_year.count()
        return render(request, self.template_name, context)



from __future__ import unicode_literals

from django.apps import AppConfig
from django.db.models.fields import Field
from handyhelpers.lookups import NotEqual


class HandyHelpersConfig(AppConfig):
    name = 'handyhelpers'
    verbose_name = "django-handyhelpers"

    def ready(self):
        # register field lookups
        Field.register_lookup(NotEqual)



import csv
from django.http import HttpResponse
from django.views.generic import View


class CsvExportView(View):
    """ dump a queryset to a csv file """
    queryset = None
    filename = None

    def get(self, request):
        try:
            model = self.queryset.model
            if not self.filename:
                self.filename = "{}.csv".format(model._meta.model_name)
            response = HttpResponse(content_type='text/csv')
            cd = 'attachment; filename="{0}"'.format(self.filename)
            response['Content-Disposition'] = cd
            headers = [field.name for field in model._meta.fields]
            writer = csv.DictWriter(response, fieldnames=headers)
            writer.writeheader()
            for row in self.queryset:
                writer.writerow({column: str(getattr(row, column)) for column in headers})
            return response
        except AttributeError:
            return HttpResponse(content_type='text/csv')



"""
Permissions classes used with DRF APIs
"""

from rest_framework.permissions import BasePermission, SAFE_METHODS


class IsAdminOrReadOnly(BasePermission):
    """ The request is authenticated as an admin, or is a read-only request. """
    def has_permission(self, request, view):
        return (
            request.method in SAFE_METHODS or
            request.user and
            request.user.is_staff
        )


class IsInAllGroups(BasePermission):
    """ 
    Description:
        Restrict access based on request method and user group; user must be in ALL required groups 
    
    Usage:
        put the following in your viewset:
            permission_classes = (IsInAllGroups,)
            required_groups = {'POST': ['site_admins'],
                               'PATCH': ['site_admins'],
                               'PUT': ['site_admins'],
                               'DELETE': ['site_admins', 'site_managers'],
                              }

    """
    def has_permission(self, request, view):
        required_groups_mapping = getattr(view, 'required_groups', {})
        required_groups = required_groups_mapping.get(request.method, [])
        if required_groups is None:
            return False
        return set(required_groups).issubset([i.name for i in request.user.groups.all()])


class IsInAnyGroup(BasePermission):
    """ 
    Description:
        Restrict access based on request method and user group; user can be in ANY required group 
    
    Usage:
        put the following in your viewset:
            permission_classes = (IsInAnyGroup,)
            required_groups = {'POST': ['site_admins'],
                               'PATCH': ['site_admins'],
                               'PUT': ['site_admins'],
                               'DELETE': ['site_admins', 'superusers'],
                              }
    """
    def has_permission(self, request, view):
        required_groups_mapping = getattr(view, 'required_groups', {})
        required_groups = required_groups_mapping.get(request.method, [])
        if required_groups is None:
            return False
        return any(group in [i.name for i in request.user.groups.all()] for group in required_groups)



import os
from setuptools import setup, find_packages
import handyhelpers


with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as readme:
    README = readme.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name='django-handyhelpers',
    description='A collection of handy utilities to support django operations',
    long_description=README,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,
    version=handyhelpers.__version__,
    license=handyhelpers.__license__,
    author=handyhelpers.__author__,
    author_email=handyhelpers.__email__,
    url='https://github.com/davidslusser/django-handyhelpers',
    download_url='https://github.com/davidslusser/django-handyhelpers/archive/{}.tar.gz'
                 ''.format(handyhelpers.__version__),
    keywords=['django', 'helpers', 'handyhelpers'],
    install_requires=required,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
